@@ -3,7 +3,7 @@
  *
  *      Copyright 2009 - 2012 Hong Jen Yee (PCMan) <pcman.tw@gmail.com>
  *      Copyright 2009 Juergen Hoetzel <juergen@archlinux.org>
- *      Copyright 2012-2014 Andriy Grytsenko (LStranger) <andrej@rep.kiev.ua>
+ *      Copyright 2012-2016 Andriy Grytsenko (LStranger) <andrej@rep.kiev.ua>
  *
  *      This file is a part of the Libfm library.
  *
@@ -251,7 +251,12 @@ gboolean _fm_file_info_set_from_native_file(FmFileInfo* fi, const char* path,
                 fi->mime_type = fm_mime_type_from_file_name(fm_path_get_basename(fi->path));
         }
         else
+        {
             fi->mime_type = fm_mime_type_from_native_file(path, fm_path_get_basename(fi->path), &st);
+            if (G_UNLIKELY(fi->mime_type == NULL))
+                /* file might be deleted while we test it but we assume mime_type is not NULL */
+                fi->mime_type = fm_mime_type_from_name("application/octet-stream");
+        }
 
         if (get_fast) /* do rough estimation */
             fi->accessible = ((st.st_mode & S_IRUSR) == S_IRUSR);
@@ -284,12 +289,18 @@ gboolean _fm_file_info_set_from_native_file(FmFileInfo* fi, const char* path,
                             FmMimeType *new_mime_type = fm_mime_type_from_file_name(uri);
 
                             /* g_debug("got type %s for URL %s", fm_mime_type_get_type(new_mime_type), uri); */
-                            if (new_mime_type == _fm_mime_type_get_application_x_desktop() ||
-                                strcmp(fm_mime_type_get_type(new_mime_type),
-                                       "application/octet-stream") == 0)
+                            if (strcmp(fm_mime_type_get_type(new_mime_type),
+                                       "application/octet-stream") == 0 ||
+                                /* actually remote links should never be
+                                   directories so let treat them as unknown */
+                                (new_mime_type == _fm_mime_type_get_inode_directory()
+                                 && !g_str_has_prefix(uri, "file:/")))
                             {
-                                /* either link to desktop entry or failed
-                                   to determine, set to inode/x-shortcut */
+                                /* NOTE: earlier we classified all links to
+                                   desktop entry as inode/x-shortcut too but
+                                   that would require a lot of special support
+                                   therefore we set to inode/x-shortcut only
+                                   those shortcuts that we fail to determine */
                                 fm_mime_type_unref(new_mime_type);
                                 new_mime_type = fm_mime_type_ref(_fm_mime_type_get_inode_x_shortcut());
                             }
