@@ -2,22 +2,23 @@
  *      fm-file-ops-job.c
  *
  *      Copyright 2009 PCMan <pcman.tw@gmail.com>
- *      Copyright 2012-2014 Andriy Grytsenko (LStranger) <andrej@rep.kiev.ua>
+ *      Copyright 2012-2015 Andriy Grytsenko (LStranger) <andrej@rep.kiev.ua>
  *
- *      This program is free software; you can redistribute it and/or modify
- *      it under the terms of the GNU General Public License as published by
- *      the Free Software Foundation; either version 2 of the License, or
- *      (at your option) any later version.
+ *      This file is a part of the Libfm library.
  *
- *      This program is distributed in the hope that it will be useful,
+ *      This library is free software; you can redistribute it and/or
+ *      modify it under the terms of the GNU Lesser General Public
+ *      License as published by the Free Software Foundation; either
+ *      version 2.1 of the License, or (at your option) any later version.
+ *
+ *      This library is distributed in the hope that it will be useful,
  *      but WITHOUT ANY WARRANTY; without even the implied warranty of
- *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *      GNU General Public License for more details.
+ *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ *      Lesser General Public License for more details.
  *
- *      You should have received a copy of the GNU General Public License
- *      along with this program; if not, write to the Free Software
- *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
- *      MA 02110-1301, USA.
+ *      You should have received a copy of the GNU Lesser General Public
+ *      License along with this library; if not, write to the Free Software
+ *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 /**
@@ -660,14 +661,37 @@ static gboolean _fm_file_ops_job_link_run(FmFileOpsJob* job)
     {
         FmPath* path = FM_PATH(l->data);
         char* src = NULL;
-        GFile* dest = g_file_get_child(dest_dir, fm_path_get_basename(path));
+        char *_basename = NULL;
+        const char *basename = fm_path_get_basename(path);
+        GFile* dest;
         GError* err = NULL;
-        char* dname;
+        char* dname = NULL;
 
+        /* if we drop URI query onto native filesystem, omit query part */
+        if (!fm_path_is_native(path) && g_file_is_native(dest_dir))
+            dname = strchr(basename, '?');
+        /* if basename consist only from query then use first part of it */
+        if (dname == basename)
+        {
+            basename++;
+            dname = strchr(basename, '&');
+        }
+        if (dname)
+        {
+            _basename = g_strndup(basename, dname - basename);
+            dname = strrchr(_basename, G_DIR_SEPARATOR);
+            g_debug("cutting '%s' to '%s'",basename,dname?&dname[1]:_basename);
+            if (dname)
+                basename = &dname[1];
+            else
+                basename = _basename;
+        }
         /* showing currently processed file. */
+        dest = g_file_get_child(dest_dir, basename);
         dname = g_file_get_parse_name(dest);
         fm_file_ops_job_emit_cur_file(job, dname);
         g_free(dname);
+        g_free(_basename);
 
 _retry_link:
         if (fm_path_is_native(path))
@@ -769,7 +793,7 @@ _link_error:
                                     "Type=Link\n"
                                     "Name=%s"
                                     "%s%s\n"
-                                    "URL=%s", name,
+                                    "URL=%s\n", name,
                                     iname ? "\nIcon=" : "", iname ? iname : "",
                                     src);
             g_free(name);
